# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_process.ipynb.

# %% auto 0
__all__ = ['ResponseProcess', 'ProcessFunction_ResponseTypeError', 'ProcessFunction_Error', 'process_function']

# %% ../nbs/00_process.ipynb 2
from dataclasses import dataclass, field
from typing import Any, Callable, List
from functools import wraps
import httpx

# %% ../nbs/00_process.ipynb 3
@dataclass
class ResponseProcess:
    function_name : str
    id : str
    message: List = field(default_factory = lambda: ['init'])
    response : Any = field(repr = False, default = None)
    is_success : bool = False,
    
    def to_json(self):
        return [ {** self.__dict__, 'message' : msg} for msg in self.message]

# %% ../nbs/00_process.ipynb 5
class ProcessFunction_ResponseTypeError(TypeError):
    def __init__(self, result):
        super().__init__(
            f"Expected function to return an instance of ResponseProcess got {type(result)} instead.  Refactor function to return ResponseGetData class"
        )

class ProcessFunction_Error(Exception):
    def __init__(self, log, auth):
        super().__init__(f"💀 | {log.function_name } | {id} | {message} | in {auth.domo_instance}")


def process_function(func: Callable[..., Any]) -> Callable[..., Any]:
    """
    Decorator for route functions to ensure they receive certain arguments.
    If these arguments are not provided, default values are used.

    Args:
        func (Callable[..., Any]): The function to decorate.

    Returns:
        Callable[..., Any]: The decorated function.

    The decorated function takes the following arguments:
        *args (Any): Positional arguments for the decorated function.
        parent_class (str, optional): The parent class. Defaults to None.
        debug_num_stacks_to_drop (int, optional): The number of stacks to drop for debugging. Defaults to 1.
        debug_api (bool, optional): Whether to debug the API. Defaults to False.
        session (httpx.AsyncClient, optional): The HTTPX client session. Defaults to None.
        **kwargs (Any): Additional keyword arguments for the decorated function.
    """

    @wraps(func)
    async def wrapper(
        *args: Any,
        auth,
        session : httpx.AsyncClient,
        debug_api : bool,
        log : List[ResponseProcess],
        **kwargs: Any,
    ) -> Any:
        result = await func(
            *args,
            auth = auth,
            debug_api=debug_api,
            session=session,
            log = log,
            **kwargs,
        )

        if not isinstance(result, ResponseProcess):
            raise ProcessFunction_ResponseTypeError(result)

        return result

    return wrapper



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_FactoryAuth.ipynb.

# %% auto 0
__all__ = ['modify_config_generate_config_auth_fn', 'modify_config_generate_admin_auth_fn', 'generate_config_auth',
           'Config_GenerateConfigAuth', 'get_jupyteraccount', 'Config_DomoJupyterToDomoAccount']

# %% ../nbs/00_FactoryAuth.ipynb 1
from .utils.factory import (
    factory_function,
    FactoryLogs, 
    FactoryResponse, FactoryConfig, FactoryMessage)

# %% ../nbs/00_FactoryAuth.ipynb 2
from dataclasses import dataclass
from typing import Callable, Optional

from copy import deepcopy

import domolibrary.client.DomoAuth as dmda
import domolibrary_execution.utils.domojupyter as dxdj

# %% ../nbs/00_FactoryAuth.ipynb 5
async def modify_config_generate_config_auth_fn(step, config,
                                                config_auth_exception : dmda.DomoAuth,
                                                config_auth_standard: dmda.DomoAuth,
                                                target_instance_use_exception_pw : int,
                                                target_instance : str,
                                                **kwargs,
                                                ):
    auth = deepcopy(config_auth_standard)
    step.message = "using standard auth"

    if target_instance_use_exception_pw == 1:
        auth = deepcopy(config_auth_exception)
        step.message = "using exception auth"

    auth.domo_instance = target_instance

    step.is_success = True
    config.auth = auth
    return config

async def modify_config_generate_admin_auth_fn(step, config,
                                                config_auth_prod : dmda.DomoAuth,
                                                config_auth_test: dmda.DomoAuth,
                                                target_instance_use_prod_pw : int,
                                                target_instance: str,
                                                **kwargs,
                                                ):
    auth = deepcopy(config_auth_prod)
    step.message = "using prod auth"

    if target_instance_use_prod_pw == 0:
        auth = deepcopy(config_auth_test)
        step.message = "using test auth"
        step.is_success = True

    auth.domo_instance= target_instance
    step.is_success = True
    config.auth = auth

    return config


@factory_function
async def generate_config_auth(
    config : FactoryConfig,
    res: FactoryResponse,
    modify_config_fn: Optional[Callable] = None,
    # ouptut
    auth : dmda.DomoAuth = None, # will be updated during execution
    debug_api: bool = False,
    
    **kwargs
):

    if modify_config_fn:
        step = FactoryMessage(stage="executing modify_config_fn", stage_num=1)
        step.is_success = False
        res.add_message(step)

        await modify_config_fn(
            **config.asdict(),
            config=config, step=step, debug_api=debug_api
        )

        auth = config.auth    

    res.location = auth.domo_instance

    try:
        step = FactoryMessage(stage="test is valid auth", stage_num = 2)
        res.add_message(step)
        await auth.print_is_token()
        step.is_success = True
        step.message = "valid auth"

    except Exception as e:
        step.message = e
        step.is_success = False

    res.response = auth
    return res


@dataclass
class Config_GenerateConfigAuth(FactoryConfig):
    modify_config_fn : Callable

    # updated during execution
    auth: dmda.DomoAuth = None
    target_instance: str = None



# %% ../nbs/00_FactoryAuth.ipynb 7
@factory_function
async def get_jupyteraccount(
    config: FactoryConfig,
    res : FactoryResponse,
    
    domojupyter_fn : Callable,
    account_name : str, # name of the domo jupyter account and account object to retrieve
    
    account_is_abstract : bool = False,
    modify_config_fn : Callable = None, # can update config


    #updated during execution
    account_creds: dict = None ,

    # used by wrapper
    **kwargs
) -> FactoryResponse:

    step = FactoryMessage(stage="retrieve domojupyter creds")
    res.add_message(step)

    try:
        account_creds = dxdj.read_domo_jupyter_account(
            account_name=account_name,
            domojupyter_fn=domojupyter_fn,
            is_abstract=account_is_abstract,
        )
        step.is_success = True
        step.message = f"domojupyter account {account_name} retrieved"

    except Exception as e:
        step.is_success = False
        step.message = f"failed to retrieve {account_name} - has it been shared with the workbook? {e}"

    if not account_creds:
        res.response = "no creds"
        return res

    config.account_creds = account_creds
    res.response = step.message

    if modify_config_fn:
        step = FactoryMessage(stage=f"modify config", is_success=False)
        res.add_message(step)
        modify_config_fn(config=config, step=step, account_creds=account_creds)

    return res


@dataclass
class Config_DomoJupyterToDomoAccount(FactoryConfig):
    account_name : str # name of the domo jupyter account and account object to retrieve
    domojupyter_fn: Callable
    
    account_is_abstract : bool = False
    
    
    modify_config_fn : Callable = None # can update config

    #updated during execution
    account_creds: dict = None 




# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_FactoryRole.ipynb.

# %% auto 0
__all__ = ['get_role', 'GetRole', 'upsert_role', 'UpsertRoleConfig']

# %% ../nbs/00_FactoryRole.ipynb 2
from .utils.factory import (
    factory_function,
    FactoryLogs, 
    FactoryResponse, FactoryConfig, FactoryMessage)

# %% ../nbs/00_FactoryRole.ipynb 3
from typing import List
from dataclasses import dataclass
import httpx

import domolibrary.client.DomoAuth as dmda

import domolibrary.classes.DomoInstanceConfig as dmic
import domolibrary.classes.DomoRole as dmdr

# %% ../nbs/00_FactoryRole.ipynb 6
@factory_function
async def get_role(
    logs: FactoryLogs,
    res: FactoryResponse,
    config: FactoryConfig,
    auth: dmda.DomoAuth,

    get_default_role: bool = False,  # if set to True will retrieve default role
    role_name: str = None, # will retrieve role searched by name
    
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
    debug_factory: bool = False,
    
    **kwargs,
):

    res.response = False

    domo_role = None

    if debug_factory:
        from pprint import pprint
        pprint(logs.logs)

    # if not role_name and not get_default_role:
    #     raise FactoryFunction_Error(
    #         factory = res,
    #         location=config.location,
    #         message="must pass `role_name` or get_default_role = True",
    #     )

    step = FactoryMessage(stage="retrieve role", is_success=False)
    res.add_message(step)

    if get_default_role:
        try:
            domo_role = await dmdr.DomoRoles.get_default_role(
                auth=auth, debug_api=debug_api, session=session
            )
            step.is_success = True
            step.message = "default role retrieved"

        except Exception as e:
            step.is_success = False
            step.message = f"unable to retrieve default role - {str(e)}"


    if not domo_role:
        try:
            domo_role = await dmdr.DomoRoles.search_role(auth=auth, role_name=role_name)
            step.is_success = True
            step.message = f"search role {role_name} retrieved {domo_role.name}"

        except Exception as e:
            step.is_success = False
            step.message = f"unable to retrieve search role {role_name} - {str(e)}"

    if not domo_role:
        res.response = "no domo role"
        return res

    config.domo_role = domo_role
    res.response = domo_role

    if debug_factory:
        pprint(logs.logs)

    return res


@dataclass
class GetRole(FactoryConfig):
    auth: dmda.DomoAuth
    role_name: str = None
    get_default_role : bool = False
    domo_role : dmdr. DomoRole = None # will update during execution


# %% ../nbs/00_FactoryRole.ipynb 10
@factory_function
async def upsert_role(
    logs: FactoryLogs,
    res : FactoryResponse,
    config : FactoryConfig,
    auth: dmda.DomoAuth,
    
    grant_ls : List[str] = None,
    is_all_grants: bool = False, # use when generating super_admin account

    role_name : str = None,
    role_description : str = None,
    set_default_role: bool = False,

    # output
    domo_role : dmdr.DomoRole = None, # gets assigned during function execution

    debug_api: bool = False,
    session : httpx.AsyncClient = None,
    **kwargs
    ):
    
    res.response = False
    
    # STAGE_1 = GET GRANTS
    step = FactoryMessage( stage="retrieve grants", is_success=False, stage_num=1)
    res.add_message( step )
    
    if grant_ls:
        step.message = "using grants from config"
    
    # STAGE_2 = USE ALL GRANTS
    if is_all_grants:
        try:
            step_all_grants = FactoryMessage( stage="get super_admin grants", is_success=False, stage_num=2)
            res.add_message( step_all_grants )

            domo_instance = dmic.DomoInstanceConfig(auth = auth)
            grant_ls = await domo_instance.get_grants(session = session, debug_api = debug_api)     
            
            config.grant_ls = grant_ls
            step_all_grants.message = "using all grants"
            step_all_grants.is_success=True
            
        except Exception as e:
            step.is_success = False
            step.message = str(e)

    if not grant_ls:
        res.response = "no grant list"
        return res
    

    step.is_success = True
            
    # STAGE_3 = upsert DOMO_ROLE
    step = FactoryMessage( stage="upsert role", is_success=False, stage_num=3)
    res.add_message( step )

    domo_role = None
    # try:
    domo_role =  await dmdr.DomoRoles.upsert_role(auth=config.auth,
                                                name=role_name,
                                                description=role_description,
                                                grant_ls=grant_ls, 
                                                session = session,
                                                debug_api = debug_api)
    
    config.domo_role = domo_role
    step.is_success = True
    step.message = f"{domo_role.name} upserted"
    
    # except Exception as e:
        # step.is_success = False
        # step.message = str(e)
    
    if not domo_role:
        res.response = "no domo role"
        return res
    
    
    # STAGE_4 = config DEFAUL_ROLE
    if set_default_role:
        step = FactoryMessage( stage="set default role", is_success=False, stage_num=4)
        res.add_message( step )
        
        try:
            await domo_role.set_as_default_role(session = session, debug_api = debug_api)
            step.is_success = True
            step.message = f"{domo_role.name} set to default role in {auth.domo_instance}"
        
        except Exception as e:
            step.is_success = False
            step.message = str(e)
        
    res.response = domo_role
    return res

@dataclass
class UpsertRoleConfig(FactoryConfig):
    auth: dmda.DomoAuth
    role_name: str
    role_description:str
    grant_ls: List[str]
    
    is_all_grants : bool = False
    set_default_role: bool = False
    
    domo_role : dmdr.DomoRole = None
    
    # factory_fn_ls : List[Callable] = field(default_factory = lambda:[sync_role])




# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/00_utils.ipynb.

# %% auto 0
__all__ = ['which_environment', 'read_domo_jupyter_account', 'domojupyter_to_domoauth']

# %% ../../nbs/00_utils.ipynb 2
import os 
import urllib.parse as urllib_parse
import json


from operator import itemgetter

import json
from typing import Callable

import domolibrary.client.DomoAuth as dmda

# %% ../../nbs/00_utils.ipynb 3
def which_environment():
    return urllib_parse.urlparse(os.environ.get('DOMO_HOSTNAME')).netloc.replace('.domo.com', '')
    

# %% ../../nbs/00_utils.ipynb 4
def read_domo_jupyter_account(account_name,
                              domojupyter_fn : Callable,
                              is_abstract: bool = False,
                              is_dict : bool = True
                            
                             ):

    account_properties = domojupyter_fn.get_account_property_keys(account_name)

    creds = {
        prop: domojupyter_fn.get_account_property_value(account_name, prop)
        for prop in account_properties
    }

    if not is_abstract:
        return creds

    creds = creds['credentials']
    
    if not is_dict:
        return creds.strip()
    
    return json.loads(
        creds
    )

# %% ../../nbs/00_utils.ipynb 5
def remove_asterix(text):
    if text == '' or text == '********':
        return None
    return text

# %% ../../nbs/00_utils.ipynb 6
async def domojupyter_to_domoauth(
    account_name,
    instance,
    domojupyter_fn: Callable,
    is_abstract: bool,
    is_test: bool = True
) -> dict:

    creds = None
      
    creds = read_domo_jupyter_account(account_name,  
                                      domojupyter_fn, 
                                      is_abstract = is_abstract)    
    

    creds = {
        "access_token" : creds.get('DOMO_ACCESS_TOKEN') if is_abstract else creds.get('domoAccessToken'),
        "password" : creds.get('DOMO_PASSWORD') if is_abstract else creds.get('password'),
        "username" :creds.get('DOMO_USERNAME') if is_abstract else creds.get('username'),
    }

    creds= { key : remove_asterix(value) for key, value in creds.items()}
    
    access_token, password, username = itemgetter('access_token', 'password', 'username')(creds)
    
    
    auth = None
    if not auth and access_token:
        auth = dmda.DomoTokenAuth(
            domo_instance = instance,
            domo_access_token = access_token
        )
        
        if is_test and not ( await auth.print_is_token()):
            auth = None
    
    if not auth and username and password:
        auth = dmda.DomoFullAuth(
            domo_instance = instance,
            domo_username = username,
            domo_password = password
        )
        
    if is_test and not ( await auth.print_is_token()):
        auth = None
    
    if not auth:
        raise Exception(f"unable to export auth from {account_name}")

                        
    return auth

